#include <octave/oct.h>
#include <octave/parse.h>

DEFUN_DLD(reduce_cpp, args, nargout,
          "-*- texinfo -*-\n"
          "@deftypefn {FunctionFile} {@var{y} = } reduce (@var{f},@var{x})\n"
          ""
          "Reduces the vector @var(x) to a scalar by repeatedly folding the\n"
          "arguments into the function @var(f) from the left.\n"
          "@end deftypefn") {
  // Check and extract the arguments
  if ( args.length() != 2 || 
       !(args(0).is_function_handle() || args(0).is_inline_function()) ||
       !args(1).is_matrix_type() ||
       !(args(1).dims().length()==2 && (args(1).dims().elem(0)==1 || args(1).dims().elem(1)==1)) ||
       args(1).length() < 2) {
    print_usage();
    return octave_value_list();
  }

  // Reduce x
  octave_function *f = args(0).function_value();
  RowVector x = args(1).row_vector_value();

  octave_value y = octave_value(x(0));
  octave_value_list fx(1,2);

  for(int i=1; i<x.length(); ++i) {
    fx(0) = y;
    fx(1) = octave_value(x(i));
    y = feval(f,fx,1)(0);
  }

  return y;
}
